dcomplex L(dcomplex N) {
  return -2.*( EulerGamma + psi0(N+1.) )/N +1./N/N;
}
dcomplex C1_pointlike_gg(dcomplex N) {
  return 4.*CA*mell_D(1,N)
    + 2*CA * ZETA2
    + CA * 2.*( L(N-1) - 2.*L(N) + L(N+1) - L(N+2) + psi_n(1,N) )
    - 11./2.*( 1/(N-1.) -3./N +3./(N+1.) -1./(N+2.) );
}
dcomplex C1_pointlike_qg(dcomplex N) {
  return 2./3.*(2*L(N-1.)-2*L(N)+L(N+1.)) -1./(N-1.) +2./N -1./3./(N+1.);
}
dcomplex C1_pointlike_qqb(dcomplex N) {
  return 32./27.*(1./(N-1.) - 3./N + 3./(N+1.) -1./(N+2.));
}


dcomplex NFO_1_gg(dcomplex N, double mH) {
  dcomplex res = 0;
  if(finite_mt) {
    double c1 = sxc->c1marzani(mH,mtop);
    if(mH<2*mtop) {
      res = gg_nlo_mt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, c1);
    } else {
      res = gg_nlo_infmt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, c1);
    }
  } else {  // pointlike
    res = C1_pointlike_gg(N) + _W1;
  }
  return res;
}
dcomplex NFO_1_qg(dcomplex N, double mH) {
  dcomplex res = 0;
  if(finite_mt) {
    double c1 = sxc->c1marzani(mH,mtop);
    double c10 = sxc->c10(mH,mtop);
    if(mH<2*mtop) {
      res = qg_nlo_mt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, (c1-c10)*CF/CA);
    } else {
      res = qg_nlo_infmt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, (c1-c10)*CF/CA);
    }
  } else {  // pointlike
    res = C1_pointlike_qg(N);
  }
  return res;
}
dcomplex NFO_1_qqb(dcomplex N, double mH) {
  dcomplex res = 0;
  if(finite_mt) {
    double c1 = sxc->c1marzani(mH,mtop);
    double c10 = sxc->c10(mH,mtop);
    if(mH<2*mtop) {
      res = qqb_nlo_mt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, (c1-2*c10)*pow(CF/CA,2));
    } else {
      res = qqb_nlo_infmt_N(N, mH, mtop, 1.) + smallx_match_nlo_N(N, (c1-2*c10)*pow(CF/CA,2));
    }
  } else {  // pointlike
    res = C1_pointlike_qqb(N);
  }
  return res;
}




dcomplex NFO_2_gg(dcomplex N, double mH, double muFmHratio=1) {
  dcomplex res;
  if(mH<2*mtop && finite_mt) {
    double lf = -2.*log(muFmHratio);
    double c1 = sxc->c1marzani(mH,mtop);
    double c2 = sxc->c2marzani(mH,mtop);
    c2  += (c1+lf)*lf*2.;
    res = gg_nnlo_mt_N(N-1., mH, mtop, muFmHratio) + smallx_match(N-1, c2);
  } else {
    //res = C2reg_pointlike_gg_softexp(N,_nf) + C2reg_pointlike_gg_smallx(N,_nf) + C2log_pointlike_gg(N,mell_D,_nf) + _cd2 + _W1*C1_pointlike_gg(N) + _W2;
    res = C2_pointlike_gg(N) + _W1*C1_pointlike_gg(N) + _W2;
    if(isPseudoscalar) res += (44*_nf-1332.)/24./N +(990.-4*_nf)/24./(N-1.) +(342.-40*_nf)/24./(N+1.) + (21.+2*_nf)/2./N/N +18./(N-1.)/(N-1.) + 2*(2*_nf-27.)/3./N/N/N;
  }
  return res;
}
dcomplex NFO_2_qg(dcomplex N, double mH, double muFmHratio=1) {
  dcomplex res;
  if(mH<2*mtop && finite_mt) {
    double lf = -2.*log(muFmHratio);
    double c10 = sxc->c10(mH,mtop);
    double c20 = sxc->c20(mH,mtop);
    double c11 = sxc->c11(mH,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qg_nnlo_mt_N(N-1, mH, mtop, muFmHratio) + smallx_match(N-1, (c2-c20)*CF/CA);
  } else {
    //res = C2reg_pointlike_qg_softexp(N,_nf) + C2reg_pointlike_qg_smallx(N,_nf) + _W1*C1_pointlike_qg(N);
    res = C2_pointlike_qg(N) + _W1*C1_pointlike_qg(N);
    if(isPseudoscalar) res += -200./9./N + 173./9./(N-1.) + 3./(N+1.) + 28./3./N/N + 8./(N-1.)/(N-1.) - 2*28./9./N/N/N;
  }
  return res;
}
dcomplex NFO_2_qqb(dcomplex N, double mH, double muFmHratio=1) {
  dcomplex res;
  if(mH<2*mtop && finite_mt) {
    double lf = -2.*log(muFmHratio);
    double c10 = sxc->c10(mH,mtop);
    double c20 = sxc->c20(mH,mtop);
    double c11 = sxc->c11(mH,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qqb_nnlo_mt_N(N-1, mH, mtop, muFmHratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else {
    //res = C2reg_pointlike_qqb_softexp(N,_nf) + C2reg_pointlike_qqb_smallx(N,_nf) + _W1*C1_pointlike_qqb(N);
    res = C2_pointlike_qqb(N) + _W1*C1_pointlike_qqb(N);
    if(isPseudoscalar) res += -16./27./N + 16*(11.-_nf)/27./(N-1.) + 16.*(_nf-9.)/27./(N+1.) -16./27./(N+2.) + 32.*(8.-_nf)/27./N/N + 32./9./(N-1.)/(N-1.) + 2*32./27./N/N/N;
  }
  return res;
}
dcomplex NFO_2_qq(dcomplex N, double mH, double muFmHratio=1) {
  dcomplex res;
  if(mH<2*mtop && finite_mt) {
    double lf = -2.*log(muFmHratio);
    double c10 = sxc->c10(mH,mtop);
    double c20 = sxc->c20(mH,mtop);
    double c11 = sxc->c11(mH,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qq_nnlo_mt_N(N-1, mH, mtop, muFmHratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else {
    //res = C2reg_pointlike_qq_softexp(N,_nf) + C2reg_pointlike_qq_smallx(N,_nf);
    res = C2_pointlike_qq(N);
    if(isPseudoscalar) res += -320./27./N + 296./27./(N-1.) + 8./9./(N+1.) + 176./27./N/N + 32./9./(N-1.)/(N-1.) - 2*64./27./N/N/N;
  }
  return res;
}
dcomplex NFO_2_qqp(dcomplex N, double mH, double muFmHratio=1) {
  dcomplex res;
  if(mH<2*mtop && finite_mt) {
    double lf = -2.*log(muFmHratio);
    double c10 = sxc->c10(mH,mtop);
    double c20 = sxc->c20(mH,mtop);
    double c11 = sxc->c11(mH,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qqp_nnlo_mt_N(N-1, mH, mtop, muFmHratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else {
    //res = C2reg_pointlike_qqp_softexp(N,_nf) + C2reg_pointlike_qqp_smallx(N,_nf);
    res = C2_pointlike_qqp(N);
    if(isPseudoscalar) res += -32./9./N + 88./27./(N-1.) + 8./27./(N+1.) + 16./3./N/N + 32./9./(N-1.)/(N-1.) - 2*16./9./N/N/N;
  }
  return res;
}


